/**
 * @author YaredFall <https://github.com/YaredFall>
 */
"use strict";
const utils = require("../utils");

module.exports = {
    meta: {
        //** @type {"problem"} */
        type: "problem",
        docs: {
            description: "enforce valid `defineProps` prop options compiler macro",
        },
        schema: [],
        messages: {
            invalidProperty: "Can not use unknow property '{{ pname }}' inside PropOption",
        },
    },
    /** @param {RuleContext} context */
    create(context) {
        const scriptSetup = utils.getScriptSetupElement(context);
        if (!scriptSetup) {
            return {};
        }

        return utils.compositingVisitors(
            utils.defineScriptSetupVisitor(context, {
                onDefinePropsEnter(node) {
                    if (node.arguments.length > 0) {
                        const define = node.arguments[0];
                        if (define.type === "ObjectExpression") {
                            define.properties.forEach((p) => {
                                const prop = p.value;
                                if (prop.type === "ObjectExpression") {
                                    prop.properties.forEach((pp) => {
                                        const pname = utils.getStaticPropertyName(pp);
                                        if (!["type", "required", "default", "validator"].includes(pname))
                                            context.report({
                                                node: pp,
                                                messageId: "invalidProperty",
                                                data: {
                                                    pname
                                                }
                                            });
                                    });
                                }
                            });
                        }
                    }
                },
            })
        );
    },
};
